#include <stdio.h>
#include <netdb.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

int create_socket(int *clientfd, int port, int clients) {
	int i;
	int yes = 1;					// Solaris reportedly uses char, all others use int
	int socketfd;
	socklen_t sin_size = sizeof(struct sockaddr_in);
	struct sockaddr_in server_address;		// server address information
	struct sockaddr_in client_address;		// client address information
	struct servent *service;

	if (!clientfd || !port || !clients) return NULL;

	if (service = getservbyport(port,"tcp")) {
		fprintf(stderr,"Error: port %d/%s reserved for use by %s\n", service->s_port, service->s_proto, service->s_name);
		return NULL;
	}

	if ((socketfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		perror("socket");
		return -1 * errno;
	}

	if (setsockopt(socketfd,SOL_SOCKET,SO_REUSEADDR,&yes,sizeof(int)) == -1) {	// Reusable socket
		perror("setsockopt");
		close(socketfd);
		return -1 * errno;
	}

	server_address.sin_family = AF_INET;			// host byte order
	server_address.sin_port = htons(port);			// short, network byte order
	server_address.sin_addr.s_addr = INADDR_ANY;		// automatically fill with my IP
	if (!memset(&(server_address.sin_zero), '\0', 8)) {	// zero the rest of the struct
		perror("memset");
		close(socketfd);
		return -1 * errno;
	}

	if (bind(socketfd, (struct sockaddr *)&server_address, sizeof(struct sockaddr)) == -1) {
		perror("bind");
		close(socketfd);
		return -1 * errno;
	}

	if (listen(socketfd, clients) == -1) {
		perror("listen");
		close(socketfd);
		return -1 * errno;
	}

	printf("Socket created.  Waiting for connections from %d clients.\n", clients);

	for (i=0;i<clients;i++) {
		if ((*(clientfd + i) = accept(socketfd, (struct sockaddr *) &client_address, &sin_size)) == -1) {
			perror("accept");
			for (--i;i>=0;i--) {
				if (!close(*(clientfd + i))) {
					perror("close");
					return -1 * errno;
				}
			}
			if (!close(socketfd)) {
				perror("close");
				return -1 * errno;
			}
			return -1 * errno;
		} else printf("Connection established from client %d of %d.\n", i+1, clients);
	}

	close(socketfd);
	return (int) clientfd;
}
