#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <unistd.h>

struct constants {
        float E;
        float I;
        float L;
};

struct variations {
        float l;
        float p;
};

static float xval[132];
static float disp[132];

void solve_system(float *, float *, float*, struct constants, struct variations);
int create_socket(int *, int, int);


int *descriptor;

int main(int argc, char **argv) {
        int port;
        int i, clients;
        int connection;

        float E;
        float I;
        float L;
        float l;
        float p;
	float deflection;

        struct constants C;
        struct variations V;

/*
 *	Parse command line arguments and create a socket
 *
 */

	for (i=1;i<argc - 1;i++) {
		if (strcmp(*(argv+i),"-p") == 0) {	/* Port switch */
			i++;
			port = atoi(*(argv + i));	/* Set port to the command line argument */
		}
		if (strcmp(*(argv+i),"-c") == 0) {	/* Clients switch */
			i++;
			clients = atoi(*(argv + i));	/* Set # of concurrent clients to command line argument */
		}
	}

	if (!port || !clients) {
		printf("Usage: %s -p port -c clients\n\n",*argv);
		printf("\t-p port\t\tport to listen for incomming clients\n");
		printf("\t-c clients\tmaximum number of concurrent clients\n");
		exit(0);
	}

	descriptor = (int *) malloc(clients * sizeof(int));
	if (create_socket(descriptor, port, clients) <= 0) {
		printf("Error: Unable to create socket\n");
		exit(0);
	}

/*
 *	Arguments Parsed, Socket Created, Code Continues...
 *
 */

	/*********** Read data from LabView  *******************/

        read(*(descriptor+0), (char*) &E, 1 * sizeof(float));
        read(*(descriptor+0), (char*) &I, 1 * sizeof(float));
        read(*(descriptor+0), (char*) &L, 1 * sizeof(float));

	/*********** Fill Constants Structure  *******************/
        C.E = E;
        C.I = I;
        C.L = L;

	/*********** Read beam data from Inventor  ******************/
        read(*(descriptor+1), (char*) &xval[0], 132 * sizeof(float));

        connection = 1;


	do {

/*********** Continually read data from LabView  *******************/
		/* Load, position  */
                read(*(descriptor + 0), (char*) &l, 1 * sizeof(float));
                read(*(descriptor + 0), (char*) &p, 1 * sizeof(float));

                printf("read position = %f, and load = %f\n", l,p);

                V.l = l;
                V.p = p;

/*********** Call physics funtion  *******************/

                solve_system(&xval[0], &disp[0], &deflection, C, V);

/*********** Continually write data to Inventor  *******************/

                write(*(descriptor+1), (char*) &l, 1 * sizeof(float));
                write(*(descriptor+1), (char*) &p, 1 * sizeof(float));
                write(*(descriptor+1), (char*) &disp[0], 132 * sizeof(float));
                write(*(descriptor+1), (char*) &deflection, sizeof(float));
		printf("sending deflection %f\n", deflection);


		read( *(descriptor+0), (char *) &connection, sizeof(int));
		write(*(descriptor+1), (char *) &connection, sizeof(int));
		write(*(descriptor+0), (char *) &connection, sizeof(int));
	} while (connection);

	for (i=0;i<clients;i++) close(*(descriptor+i));

	return 0;
}


void solve_system(float* x, float *y, float *d, struct constants C, struct variations V){
        int i, n;

        float X, Y;
        float E, I, L, p, l;
        float coeff;
        float dirac, diff;
        float paren;
	
        n = 132;


        E = C.E;
        I = C.I;
        L = C.L;
        p = V.p;
        l = V.l;
        coeff = p/(6*E*I);
        coeff *= -1.0;

        for(i = 0; i < n; i++) {
                X = *(x+ i);
                diff = X-l;
                if(diff <= 0.0) {
                        dirac = 0.0;
                } else {
                        dirac = 3.*diff;
                }
                paren = dirac;
                paren -= (X*X*X/6.0);
                paren += (X*X*L/2.0);
                Y = coeff * paren;
                *(y + i) = Y;

        }
	X = l;
	diff = X - l;
	dirac = 3.0*diff;
	paren = dirac;
	paren -= (X*X*X / 6.0);
	paren += (X*X*L/2.0);
	Y = coeff * paren;
	*d = Y;
	
}

