#define NETWORK 1
#define STANDALONE 0
#define CURRENT NETWORK
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <malloc.h>

#include <Inventor/nodes/SoCube.h>
#include <Inventor/SoDB.h>
#include <Inventor/SoInput.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/SoXtRenderArea.h>
#include <Inventor/Xt/SoXtComponent.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoQuadMesh.h>

#include "beam.dat"

int make_connection(char *, int);
void TimerCallback(void *, SoSensor *);
SoSeparator* makeBeam(void );

int descriptor;
SoTransform             *arrowTransform;
SoCoordinate3 *myCoords;

float x[132];
float y[132];
float Y0[132];


int main(int argc, char **argv) {


        int i, port = 0;
        char server[100];

/*
 *      Parse command line arguments and create a socket
 *
 */

#if CURRENT
        for (i=1;i<argc - 1;i++) {
                if (strcmp(*(argv+i),"-s") == 0) {      // Server switch
                        i++;
                        strcpy(server,*(argv + i));     // Set server to the command line argument
                }
                if (strcmp(*(argv+i),"-p") == 0) {      // Port switch
                        i++;
                        port = atoi(*(argv + i));       // Set port to the command line argument
                }
        }

        if (!server || port == 0) {
                printf("Usage: %s -s server -p port\n",*argv);
                printf("\t-s server\tserver to connect to on specified port\n");
                printf("\t-p port\t\tport to listen for incomming clients\n");
                exit(0);
        }

        if ((descriptor = make_connection(server, port)) <= 0) {
                printf("Error:  Unable to connect to %s:%d\n",server,port);
                exit(0);
        } else printf("descriptor = %d\n", descriptor);
#endif

/*
 *      Arguments Parsed, Socket Created, Code Continues...
 *
 */

        /* Initialize Inventor and Xt; create new viewer */
        Widget window = SoXt::init(*argv);
        SoXtExaminerViewer *viewer = new SoXtExaminerViewer(window);


	SoInput 		*mySceneInput;
	SoSeparator 		*root;
	SoSeparator 		*beam;


/* Instantiate Scene's Root node */
	root			= new SoSeparator;

/* Instantiate Input Node to read in iv file */
	mySceneInput		= new SoInput;


/* Copy rod coordinates */
	for(i = 0; i < 132; i++) {
		x[i] = vertexPositions[i][0]; 
		Y0[i] = vertexPositions[i][1]; 
	}

#if CURRENT
/* Write to the server the beam coordinates */
        write(descriptor, (char *) &x[0], 132*sizeof(float));
#endif

	printf("did the write... moved on\n"); fflush(stdout);
/* Open the Gview file and read its contents */ 
	mySceneInput->openFile("gview.iv");
		root = SoDB::readAll(mySceneInput);
	mySceneInput->closeFile();

/* Create pointer to gview node  */
	arrowTransform = (SoTransform *) SoNode::getByName("arrow");

/* Create time dependent callback  */
        SoTimerSensor *si = new SoTimerSensor(TimerCallback, arrowTransform);
        si->schedule();

/* Call beam function  */
	beam = makeBeam();

	if(!beam){printf("no beam\n");};
	root->addChild(beam);	

/*Add scene to the viewer, tell it what to show   */
	viewer->setSceneGraph(root);
	viewer->show();

/* Display main window  */
	SoXt::show(window); 

/* Main Inventor loop  */
	SoXt::mainLoop();      

}




/*TimerCallBack function  */
void TimerCallback(void *data, SoSensor *sensor)
{
	int i, connection;
	float position, load, fac;
	float deflection;
	float shift;

#if CURRENT
/* Read from the server the new values  */
        read(descriptor, (char *) &position, 1*sizeof(float));
        read(descriptor, (char *) &load, 1*sizeof(float));
        read(descriptor, (char *) &y[0], 132*sizeof(float));
        read(descriptor, (char *) &deflection, sizeof(float));

	for(i = 0; i < 132; i++) {
		vertexPositions[i][1] = Y0[i] - y[i] ; 
	}

/* Stretch the arrow to follow the beam, just some fudging here and there */
	fac = (1.0 + load/ 5000000.0);
	arrowTransform->scaleFactor.setValue( 1.,fac , 1.0);
	printf("def = %f\n", deflection);
	shift = -1.0 * deflection;
 
/* Move the arrow to current load location along the beam */
	arrowTransform->translation.setValue( position, shift, 0.0);
	
/* Redefine the beams coordinates with those calculated by the server */
   	myCoords->point.setValues(0, 132, vertexPositions);
        read(descriptor, (char *) &connection, sizeof(int));
	printf("I got connection = %d\n", connection); fflush(stdout);
	if(connection == 0) exit(0);

#endif
/* Reschedule the callback function  */
        sensor->schedule();

}




SoSeparator *
makeBeam()
{
/* Create Local Separator node  */
   SoSeparator *result = new SoSeparator;
   result->ref();

/* Create the Material node and assign properties  */
   SoMaterial *myMaterial = new SoMaterial;
   myMaterial->diffuseColor.setValue(.78, .57, .11);
   result->addChild(myMaterial);

/* Create SoCoordinate3 Node and define vertices  */
   myCoords = new SoCoordinate3;
   myCoords->point.setValues(0, naround*nup, vertexPositions);
   result->addChild(myCoords);

/* Define the Quad Mesh  */
   SoQuadMesh *myQuadMesh = new SoQuadMesh;
   myQuadMesh->verticesPerRow = nup;

   myQuadMesh->verticesPerColumn = naround;
   result->addChild(myQuadMesh);

   result->unrefNoDelete();
   return result;
}


